@@ -2,9 +2,11 @@ from .protocol import *
 from .commands import *
 from .parameters import *
 from .messages import *
+from .plugin_messages import *
 
 try:
     from ixcom_internal.parameters import *
     from ixcom_internal.messages import *
+    from ixcom_internal.plugin_messages import *
 except ImportError as e:
     pass
