@@ -1,8 +1,28 @@
-from .protocol import DefaultParameterPayload, Message, PayloadItem, parameter
+import os
+import json
+from .protocol import Message, PayloadItem, parse_parameter_json_folder
+from .protocol import DefaultParameterPayload, parameter
+from .protocol import DefaultPluginParameterPayload, plugin_parameter
+
+def handle_undefined_parameter(*args):
+    pass
+
+"""
+PLUGIN PARAMETER
+"""
+### ride_control plugin
+@plugin_parameter(10)
+class PARPLUGIN_IATANGLIM_Payload(DefaultPluginParameterPayload):
+    plugin_parameter_payload = Message([
+        PayloadItem(name = 'angLimLowMain', dimension = 4, datatype = 'f'),
+        PayloadItem(name = 'angLimUpMain', dimension = 4, datatype = 'f'),
+        PayloadItem(name = 'angLimLowAux', dimension = 4, datatype = 'f'),
+        PayloadItem(name = 'angLimUpAux', dimension = 4, datatype = 'f')
+    ])
 
 
 """
-PARSYS
+MANUAL AND VARIABLE LENGTH PARAMETER
 """
 class PARSYS_STRING_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -111,23 +131,19 @@ class PARSYS_CONFIGCRC_Payload(DefaultParameterPayload):
     ])
 
 @parameter(18)
-class PARSYS_OSVERSION_Payload(PARSYS_STRING64_Payload):
+class PARSYS_OSVERSION_Payload(PARSYS_STRING_Payload):
     pass
 
 @parameter(19)
 class PARSYS_SYSNAME_Payload(PARSYS_STRING64_Payload):
     pass
 
-"""
-PARIMU
-"""
 @parameter(105)
 class PARIMU_MISALIGN_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'rpy', dimension = 3, datatype = 'f'),
     ])
 
-
 @parameter(112)
 class PARIMU_BANDSTOP_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -142,14 +158,6 @@ class PARIMU_RANGE_Payload(DefaultParameterPayload):
         PayloadItem(name = 'range_gyro', dimension = 1, datatype = 'f'),
     ])
 
-
-
-"""
-PARGNSS
-"""
-
-
-
 @parameter(203)
 class PARGNSS_LATENCY_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -164,8 +172,8 @@ class PARGNSS_ANTOFFSET_Payload(DefaultParameterPayload):
         PayloadItem(name = 'stdDev', dimension = 3, datatype = 'f'),
     ])
 
-    def get_name(self):
-        return super().get_name() + '_' + str(self.data.get('reserved_paramheader', ''))
+    def get_ant_number(self):
+        return self.data.get('reserved_paramheader', '')
 
 @parameter(212)
 class PARGNSS_LOCKOUTSYSTEM_Payload(DefaultParameterPayload):
@@ -175,7 +183,6 @@ class PARGNSS_LOCKOUTSYSTEM_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved3', dimension = 1, datatype = 'H'),
     ])
 
-
 @parameter(213)
 class PARGNSS_RTCMV3CONFIG_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -185,8 +192,6 @@ class PARGNSS_RTCMV3CONFIG_Payload(DefaultParameterPayload):
         PayloadItem(name = 'baud', dimension = 1, datatype = 'I'),
     ])
 
-
-
 @parameter(217)
 class PARGNSS_MODEL_Payload(DefaultParameterPayload):
     __sub_payload = Message(
@@ -228,10 +233,7 @@ class PARGNSS_HDGOFFSET_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name='offset', dimension=1, datatype='f'),
     ])
-      
-"""
-PARMAG
-"""
+
 @parameter(300)
 class PARMAG_COM_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -246,7 +248,6 @@ class PARMAG_MISALIGN_Payload(DefaultParameterPayload):
         PayloadItem(name = 'rpy', dimension = 3, datatype = 'f'),
     ])
 
-
 @parameter(307)
 class PARMAG_CAL_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -255,31 +256,24 @@ class PARMAG_CAL_Payload(DefaultParameterPayload):
         PayloadItem(name = 'valid', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(308)
 class PARMAG_CALSTATE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'calstate', dimension = 1, datatype = 'i'),
     ])
 
-
 @parameter(309)
 class PARMAG_FOM_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'FOM', dimension = 1, datatype = 'f'),
     ])
 
-
-"""
-PARMADC
-"""
 @parameter(400)
 class PARMADC_ENABLE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'enable', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(401)
 class PARMADC_LEVERARM_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -287,10 +281,6 @@ class PARMADC_LEVERARM_Payload(DefaultParameterPayload):
         PayloadItem(name = 'leverArmStdDev', dimension = 3, datatype = 'f'),
     ])
 
-
-"""
-PARREC
-"""
 @parameter(600)
 class PARREC_CONFIG_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -299,14 +289,12 @@ class PARREC_CONFIG_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved2', dimension = 1, datatype = 'H'),
     ])
 
-
 @parameter(603)
 class PARREC_START_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'str', dimension = 128, datatype = 's'),
     ])
 
-
 @parameter(604)
 class PARREC_STOP_Payload(DefaultParameterPayload):
     pass
@@ -317,16 +305,12 @@ class PARREC_SUFFIX_Payload(DefaultParameterPayload):
         PayloadItem(name = 'suffix', dimension = 128, datatype = 's'),
     ])
 
-
 @parameter(607)
 class PARREC_DISKSPACE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'freespace', dimension = 1, datatype = 'd'),
     ])
-        
-"""
-PAREKF
-"""
+
 @parameter(700)
 class PAREKF_ALIGNMODE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -408,7 +392,6 @@ class PAREKF_ZUPT_Payload(DefaultParameterPayload):
         PayloadItem(name = 'autoZupt', dimension = 1, datatype = 'B'),
     ])
 
-
 @parameter(714)
 class PAREKF_DEFPOS_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -417,13 +400,111 @@ class PAREKF_DEFPOS_Payload(DefaultParameterPayload):
         PayloadItem(name = 'alt', dimension = 1, datatype = 'f'),
     ])
 
-
 @parameter(715)
 class PAREKF_DEFHDG_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'hdg', dimension = 1, datatype = 'f'),
     ])
 
+@parameter(717)
+class PAREKF_POWERDOWN_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'savestate', dimension = 1, datatype = 'I'),
+    ])
+
+@parameter(719)
+class PAREKF_STOREDPOS_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'lon', dimension = 1, datatype = 'd'),
+        PayloadItem(name = 'lat', dimension = 1, datatype = 'd'),
+        PayloadItem(name = 'alt', dimension = 1, datatype = 'd'),
+        PayloadItem(name = 'stdDevLon', dimension = 1, datatype = 'd'),
+        PayloadItem(name = 'stdDevLat', dimension = 1, datatype = 'd'),
+        PayloadItem(name = 'stdDevAlt', dimension = 1, datatype = 'd'),
+    ])
+
+@parameter(720)
+class PAREKF_ALIGNZUPTSTDDEV_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'zuptStdDev', dimension = 1, datatype = 'd'),
+    ])
+
+@parameter(721)
+class PAREKF_POSAIDSTDDEVTHR_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'thr', dimension = 1, datatype = 'd'),
+    ])
+
+@parameter(723)
+class PAREKF_STOREDATT_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'rpy', dimension = 3, datatype = 'f'),
+        PayloadItem(name = 'stdDev', dimension = 3, datatype = 'f'),
+    ])
+
+@parameter(724)
+class PAREKF_ODOMETER_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'sfError', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'sfStdDev', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'rwScalefactor', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'misalignmentY', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'misalignmentZ', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'stdMisalignmentY', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'stdMisalignmentZ', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'rwMisalignmentY', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'rwMisalignmentZ', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'minVel', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'maxVel', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'useAvgInno', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'enableCoarseCal', dimension = 1, datatype = 'H'),
+    ])
+
+@parameter(725)
+class PAREKF_ODOBOGIE_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'distance', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'enable', dimension = 1, datatype = 'I'),
+    ])
+
+@parameter(726)
+class PAREKF_GNSSLEVERARMEST_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'primary', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'secondary', dimension = 1, datatype = 'H'),
+    ])
+
+@parameter(727)
+class PAREKF_GNSSAIDINGRATE_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'psrpos', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'psrvel', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'rtk', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'rtktimeout', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'hdg', dimension = 1, datatype = 'H'),
+        PayloadItem(name = 'duringzupt', dimension = 1, datatype = 'H'),
+    ])
+
+@parameter(728)
+class PAREKF_KINALIGNTHR_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'thr', dimension = 1, datatype = 'f'),
+    ])
+
+@parameter(729)
+class PAREKF_PDOPTHR_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'thr', dimension = 1, datatype = 'f'),
+    ])
+
+@parameter(730)
+class PAREKF_DUALANTAID_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'thrHdg', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'thrPitch', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'thrINSHdg', dimension = 1, datatype = 'f'),
+        PayloadItem(name = 'mode', dimension = 1, datatype = 'I'),
+    ])
 
 @parameter(731)
 class PAREKF_STARTUPV2_Payload(DefaultParameterPayload):
@@ -445,117 +526,6 @@ class PAREKF_STARTUPV2_Payload(DefaultParameterPayload):
         PayloadItem(name = 'autoRestart', dimension = 1, datatype = 'B'),
     ])
 
-@parameter(717)
-class PAREKF_POWERDOWN_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'savestate', dimension = 1, datatype = 'I'),
-    ])
-
-@parameter(719)
-class PAREKF_STOREDPOS_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'lon', dimension = 1, datatype = 'd'),
-        PayloadItem(name = 'lat', dimension = 1, datatype = 'd'),
-        PayloadItem(name = 'alt', dimension = 1, datatype = 'd'),
-        PayloadItem(name = 'stdDevLon', dimension = 1, datatype = 'd'),
-        PayloadItem(name = 'stdDevLat', dimension = 1, datatype = 'd'),
-        PayloadItem(name = 'stdDevAlt', dimension = 1, datatype = 'd'),
-    ])
-
-
-@parameter(723)
-class PAREKF_STOREDATT_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'rpy', dimension = 3, datatype = 'f'),
-        PayloadItem(name = 'stdDev', dimension = 3, datatype = 'f'),
-    ])
-
-
-@parameter(720)
-class PAREKF_ALIGNZUPTSTDDEV_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'zuptStdDev', dimension = 1, datatype = 'd'),
-    ])
-
-
-@parameter(721)
-class PAREKF_POSAIDSTDDEVTHR_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'thr', dimension = 1, datatype = 'd'),
-    ])
-
-
-@parameter(724)
-class PAREKF_ODOMETER_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'sfError', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'sfStdDev', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'rwScalefactor', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'misalignmentY', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'misalignmentZ', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'stdMisalignmentY', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'stdMisalignmentZ', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'rwMisalignmentY', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'rwMisalignmentZ', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'minVel', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'maxVel', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'useAvgInno', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'enableCoarseCal', dimension = 1, datatype = 'H'),
-    ])
-
-
-@parameter(725)
-class PAREKF_ODOBOGIE_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'distance', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'enable', dimension = 1, datatype = 'I'),
-    ])
-
-
-@parameter(726)
-class PAREKF_GNSSLEVERARMEST_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'primary', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'secondary', dimension = 1, datatype = 'H'),
-    ])
-
-
-@parameter(727)
-class PAREKF_GNSSAIDINGRATE_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'psrpos', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'psrvel', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'rtk', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'rtktimeout', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'hdg', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'duringzupt', dimension = 1, datatype = 'H'),
-    ])
-
-
-@parameter(728)
-class PAREKF_KINALIGNTHR_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'thr', dimension = 1, datatype = 'f'),
-    ])
-
-
-@parameter(729)
-class PAREKF_PDOPTHR_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'thr', dimension = 1, datatype = 'f'),
-    ])
-
-
-@parameter(730)
-class PAREKF_DUALANTAID_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'thrHdg', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'thrPitch', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'thrINSHdg', dimension = 1, datatype = 'f'),
-        PayloadItem(name = 'mode', dimension = 1, datatype = 'I'),
-    ])
-
-
 @parameter(732)
 class PAREKF_MAGATTAID_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -569,7 +539,6 @@ class PAREKF_MAGATTAID_Payload(DefaultParameterPayload):
         PayloadItem(name = 'magFieldStdDev', dimension = 3, datatype = 'f'),
     ])
 
-
 @parameter(733)
 class PAREKF_MADCAID_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -585,7 +554,6 @@ class PAREKF_MADCAID_Payload(DefaultParameterPayload):
         PayloadItem(name = 'rwBias', dimension = 1, datatype = 'f'),
     ])
 
-
 @parameter(734)
 class PAREKF_ALIGNMENT_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -601,8 +569,6 @@ class PAREKF_ALIGNMENT_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved3', dimension = 3, datatype = 'I'),
     ])
 
-
-
 @parameter(737)
 class PAREKF_ZARU_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -617,9 +583,6 @@ class PAREKF_ZUPTCALIB_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved2', dimension = 1, datatype = 'H'),
     ])
 
-"""
-PARDAT
-"""
 @parameter(800)
 class PARDAT_POS_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -627,43 +590,24 @@ class PARDAT_POS_Payload(DefaultParameterPayload):
         PayloadItem(name = 'altMode', dimension = 1, datatype = 'H'),
     ])
 
-
 @parameter(801)
 class PARDAT_VEL_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'velMode', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(802)
 class PARDAT_IMU_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'imuMode', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(803)
 class PARDAT_SYSSTAT_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'statMode', dimension = 1, datatype = 'I'),
     ])
 
-
-class PARDAT_STATFPGA_Payload(DefaultParameterPayload):
-    parameter_payload = Message([
-        PayloadItem(name = 'powerStatLower', dimension = 1, datatype = 'I'),
-        PayloadItem(name = 'powerStatUpper', dimension = 1, datatype = 'I'),
-        PayloadItem(name = 'fpgaStatus', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'supervisorStatus', dimension = 1, datatype = 'H'),
-        PayloadItem(name = 'imuStatus', dimension = 1, datatype = 'B'),
-        PayloadItem(name = 'tempStatus', dimension = 1, datatype = 'B'),
-        PayloadItem(name = 'reserved2', dimension = 1, datatype = 'H'),
-    ])
-
-
-"""
-PARXCOM
-"""
 @parameter(902)
 class PARXCOM_SERIALPORT_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -673,9 +617,8 @@ class PARXCOM_SERIALPORT_Payload(DefaultParameterPayload):
         PayloadItem(name = 'baudRate', dimension = 1, datatype = 'I'),
     ])
 
-    def get_name(self):
-        return super().get_name() + '_COM' + str(self.data.get('port'))
-
+    def get_port_number(self):
+        return self.data.get('port')
 
 @parameter(903)
 class PARXCOM_NETCONFIG_Payload(DefaultParameterPayload):
@@ -690,16 +633,19 @@ class PARXCOM_NETCONFIG_Payload(DefaultParameterPayload):
         PayloadItem(name = 'gateway', dimension = 1, datatype = 'I'),
     ])
 
-    def get_name(self):
-        if 0 == self.data.get('interface'):
-            return super().get_name() + '_ETH'
-        else:
-            return super().get_name() + '_USB'
+    def get_name(self = None):
+        if self:
+            if hasattr(self,'data'):
+                if 0 == self.data.get('interface'):
+                    return super().get_name() + '_ETH'
+                else:
+                    return super().get_name() + '_USB'
+        return 'PARXCOM_NETCONFIG'
 
 @parameter(905)
 class PARXCOM_LOGLIST_Payload(DefaultParameterPayload):
     __sub_payload = Message([
-        PayloadItem(name = f'divider', dimension = 1, datatype = 'H'), 
+        PayloadItem(name = f'divider', dimension = 1, datatype = 'H'),
         PayloadItem(name = f'msgid', dimension = 1, datatype = 'H')
         ])
 
@@ -707,7 +653,6 @@ class PARXCOM_LOGLIST_Payload(DefaultParameterPayload):
         PayloadItem(name = 'logs', dimension = 16, datatype = __sub_payload)
     ])
 
-
 @parameter(906)
 class PARXCOM_AUTOSTART_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -717,7 +662,6 @@ class PARXCOM_AUTOSTART_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved2', dimension = 1, datatype = 'H'),
     ])
 
-
 @parameter(907)
 class PARXCOM_NTRIP_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -731,7 +675,6 @@ class PARXCOM_NTRIP_Payload(DefaultParameterPayload):
         PayloadItem(name = 'gga_send_period', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(908)
 class PARXCOM_POSTPROC_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -760,7 +703,7 @@ class PARXCOM_DEFAULTIP_Payload(DefaultParameterPayload):
 @parameter(917)
 class PARXCOM_LOGLIST2_Payload(DefaultParameterPayload):
     __sub_payload = Message([
-            PayloadItem(name = f'divider', dimension = 1, datatype = 'H'), 
+            PayloadItem(name = f'divider', dimension = 1, datatype = 'H'),
             PayloadItem(name = f'msgid', dimension = 1, datatype = 'H'),
             PayloadItem(name = f'running', dimension = 1, datatype = 'B'),
             PayloadItem(name = f'reserved2', dimension = 1, datatype = 'B'),
@@ -770,13 +713,12 @@ class PARXCOM_LOGLIST2_Payload(DefaultParameterPayload):
         PayloadItem(name = 'loglist', dimension = 16, datatype = __sub_payload)
     ])
 
-    def get_name(self):
-        return super().get_name() + '_Channel' + str(self.data.get('reserved_paramheader'))
+    def get_name(self = None):
+        if self:
+            if hasattr(self,"data"):
+                return super().get_name() + '_Channel' + str(self.data.get('reserved_paramheader'))
+        return "PARXCOM_LOGLIST2"            
 
-
-"""
-PARODO
-"""
 @parameter(1100)
 class PARODO_SCF_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -785,14 +727,12 @@ class PARODO_SCF_Payload(DefaultParameterPayload):
         PayloadItem(name = 'selection', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(1101)
 class PARODO_TIMEOUT_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'timeout', dimension = 1, datatype = 'f'),
     ])
 
-
 @parameter(1102)
 class PARODO_MODE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -802,28 +742,24 @@ class PARODO_MODE_Payload(DefaultParameterPayload):
         PayloadItem(name = 'deglitcherB', dimension = 1, datatype = 'H'),
     ])
 
-
 @parameter(1103)
 class PARODO_LEVERARM_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'leverArm', dimension = 3, datatype = 'f'),
     ])
 
-
 @parameter(1104)
 class PARODO_VELSTDDEV_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'stdDev', dimension = 1, datatype = 'f'),
     ])
 
-
 @parameter(1105)
 class PARODO_DIRECTION_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'direction', dimension = 3, datatype = 'f'),
     ])
 
-
 @parameter(1106)
 class PARODO_CONSTRAINTS_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -833,14 +769,12 @@ class PARODO_CONSTRAINTS_Payload(DefaultParameterPayload):
         PayloadItem(name = 'stdDev', dimension = 1, datatype = 'f'),
     ])
 
-
 @parameter(1107)
 class PARODO_RATE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'rate', dimension = 1, datatype = 'f'),
     ])
 
-
 @parameter(1108)
 class PARODO_THR_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -848,27 +782,18 @@ class PARODO_THR_Payload(DefaultParameterPayload):
         PayloadItem(name = 'thrOmg', dimension = 1, datatype = 'f'),
     ])
 
-
-
-
-
-"""
-PARARINC
-"""
 @parameter(1200)
 class PARARINC825_PORT_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'port', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(1201)
 class PARARINC825_BAUD_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'baud', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(1202)
 class PARARINC825_ENABLE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -879,8 +804,8 @@ class PARARINC825_ENABLE_Payload(DefaultParameterPayload):
 @parameter(1204)
 class PARARINC825_LOGLIST_Payload(DefaultParameterPayload):
     __sub_payload = Message([
-            PayloadItem(name = f'divider', dimension = 1, datatype = 'H'), 
-            PayloadItem(name = f'reserved', dimension = 1, datatype = 'H'), 
+            PayloadItem(name = f'divider', dimension = 1, datatype = 'H'),
+            PayloadItem(name = f'reserved', dimension = 1, datatype = 'H'),
             PayloadItem(name = f'docnumber', dimension = 1, datatype = 'I')
             ])
 
@@ -888,7 +813,6 @@ class PARARINC825_LOGLIST_Payload(DefaultParameterPayload):
         PayloadItem(name = 'logs', dimension = 30, datatype = __sub_payload)
     ])
 
-
 @parameter(1205)
 class PARARINC825_BUSRECOVERY_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -912,18 +836,12 @@ class PARARINC825_SCALEFACTOR_Payload(DefaultParameterPayload):
         PayloadItem(name = 'ScfSideSlip', dimension = 1, datatype = 'd'),
     ])
 
-
 @parameter(1208)
 class PARARINC825_EVENTMASK_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'eventMask', dimension = 1, datatype = 'I'),
     ])
 
-
-
-"""
-PARNMEA
-"""
 @parameter(1300)
 class PARNMEA_COM_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -933,8 +851,11 @@ class PARNMEA_COM_Payload(DefaultParameterPayload):
         PayloadItem(name = 'baud', dimension = 1, datatype = 'I'),
     ])
 
-    def get_name(self):
-        return super().get_name() + '_COM' + str(self.data.get('port'))
+    def get_name(self = None):
+        if self:
+            if hasattr(self,"data"):
+                return super().get_name() + '_COM' + str(self.data.get('port'))
+        return "PARNMEA_COM"
 
 
 @parameter(1301)
@@ -946,7 +867,6 @@ class PARNMEA_ENABLE_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved4', dimension = 1, datatype = 'B'),
     ])
 
-
 @parameter(1302)
 class PARNMEA_TXMASK_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -954,7 +874,6 @@ class PARNMEA_TXMASK_Payload(DefaultParameterPayload):
         PayloadItem(name = 'txMaskUDP', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(1303)
 class PARNMEA_DECPLACES_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -963,7 +882,6 @@ class PARNMEA_DECPLACES_Payload(DefaultParameterPayload):
         PayloadItem(name = 'reserved2', dimension = 1, datatype = 'H'),
     ])
 
-
 @parameter(1304)
 class PARNMEA_RATE_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -971,7 +889,6 @@ class PARNMEA_RATE_Payload(DefaultParameterPayload):
         PayloadItem(name = 'divisorUDP', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(1305)
 class PARNMEA_UDP_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -988,6 +905,17 @@ class PARNMEA_VTGSELECT_Payload(DefaultParameterPayload):
         PayloadItem(name = 'Selector', dimension = 1, datatype = 'I'),
     ])
 
+@parameter(1307)
+class PARNMEA_PRECISION_Payload(DefaultParameterPayload):
+    parameter_payload = Message([
+        PayloadItem(name = 'messageSelect', dimension = 1, datatype = 'B'),
+        PayloadItem(name = 'latitudePrecision', dimension = 1, datatype = 'B'),
+        PayloadItem(name = 'longitudePrecision', dimension = 1, datatype = 'B'),
+        PayloadItem(name = 'undulationPrecision', dimension = 1, datatype = 'B'),
+        PayloadItem(name = 'diffagePrecision', dimension = 1, datatype = 'B'),
+        PayloadItem(name = 'Reserved', dimension = 3, datatype = 'B'),
+    ])
+
 @parameter(1402)
 class PARARINC429_CFG_Payload(DefaultParameterPayload):
     parameter_payload = Message([
@@ -996,18 +924,22 @@ class PARARINC429_CFG_Payload(DefaultParameterPayload):
         PayloadItem(name = 'High Speed', dimension = 1, datatype = 'B'),
     ])
 
-"""
-IO
-"""
 @parameter(1500)
 class PARIO_HW245_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'configIO', dimension = 1, datatype = 'I'),
     ])
 
-
 @parameter(1501)
 class PARIO_HW288_Payload(DefaultParameterPayload):
     parameter_payload = Message([
         PayloadItem(name = 'toDef', dimension = 1, datatype = 'I'),
     ])
+
+
+"""
+PARAMETER FROM JSON FILES
+"""
+path_json = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'json-files', 'parameters')
+parse_parameter_json_folder(path_json)
+
